//von Bertalanffy growth model with time-varying asymptotic length
//For ADMB Short Course 1, August 2007
//Based on code by M. Wilberg for yellow perch in Lake Michigan
//Modified by B. Linton

DATA_SECTION
//Read in data from "growth_time.dat"
  init_int fyear  //first year
  init_int lyear  //last year
  init_int fage  //first age
  init_int lage  //last age
  init_matrix obs_length(fyear,lyear,fage,lage)  //observed mean length-at-age
  //ratio of mean length-at-age variance to asymptotic length variance
  init_number lambda

  init_vector test(1,3)  //data reading test values

//Define looping variables in the data section, so you only have to define them once.
//Because they are in the data section they are valid everywhere in the model.
  int i;  //looping variable for year
  int j;  //looping variable for age

//Check that data read in correctly
//!!cout << test << endl;
//!!exit(13);

PARAMETER_SECTION
//Define parameters to be estimated in model
  init_number log_length_at_fage  //mean length at first age (log scale)
  init_number log_Linf  //initial asymptotic length (log scale)
  init_number log_K  //Brody growth coefficient (log scale)
  init_vector log_init_length(fage+1,lage-1)  //mean length-at-age in the first year (log scale), ages 2-9
  init_vector log_Linf_devs(fyear+1,lyear-1)  //annual deviations in asymptotic length (log scale)

//Define quantities to be calculated from parameters
  vector Linf(fyear+1,lyear)  //vector of time-varying asymptotic lengths
  number K  //Brody growth coefficient
  matrix pred_length(fyear,lyear,fage,lage)  //matrix of predicted mean length-at-age
  number rss_length  //residual sums of squares for mean length-at-age
  number rss_Linf  //residual sums of squares for asymptotic length deviations
  number n_length  //number of mean length-at-age observations
  number n_Linf_devs  //number of asymptotic length deviations
  
//Value for the objective function to minimize
  objective_function_value c_like    //weighted negative log concentrated likelihood value

INITIALIZATION_SECTION
//Starting values for parameters
  log_Linf 3.4
  log_K -1.0
  log_length_at_fage 2.5

PRELIMINARY_CALCS_SECTION
//Calculate number of observations for concentrated likelihood
  //function size_count(x) = number of elements in object x
  n_length=size_count(pred_length);  //for mean length-at-age component
  n_Linf_devs=size_count(log_Linf_devs);  //for asymptotic length deviations component

PROCEDURE_SECTION
  get_Linf();  //Call function "get_Linf"
  get_length_at_age();  //Call function "get_length_at_age"
  evaluate_objective_function();  //Call function "evaluate_the_objective_function"

//Define functions used above
FUNCTION get_Linf
//This function estimates time-varying asymptotic lengths

  Linf(fyear+1)=exp(log_Linf);  //backtransform log scale parameter
     
//Loop fills in annual asymptotic length values according to random walk
//In random walk, Linf in time t+1 = Linf in time t * random deviation
  for (i=fyear+1;i<=lyear-1;i++)
  {
    Linf(i+1)=exp(log(Linf(i))+log_Linf_devs(i));  
  }

FUNCTION get_length_at_age
//This function estimates mean length-at-age

//Loop fills in predicted mean length-at-age matrix
  for (i=fyear;i<=lyear;i++)
  {
    //Fill in first age mean lengths
    pred_length(i,fage)=exp(log_length_at_fage);
  }

//Loop fills in initial mean lengths for ages 2-9
  for (j=fage+1;j<=lage-1;j++)
  {
    //Fill in mean lengths for ages 2 to 9 in the first year
    pred_length(fyear,j)=exp(log_init_length(j));
  }     

//Fill in the last initial age (little data to fit model)  
  pred_length(fyear,lage)=exp(log_Linf); 

//Backtransform parameter
  K=exp(log_K);

//Double loop to fill in predicted mean length-at-age matrix
  for (i=fyear;i<lyear;i++)
  {
    for (j=fage;j<lage;j++)
    {
      //von Bertalanffy growth model
      //L(t+1,a+1) = L(t,a)+(Linf(t)-L(t,a))*(1-exp(-K))
      pred_length(i+1,j+1)=pred_length(i,j)+(Linf(i+1)-pred_length(i,j))*(1-exp(-K));
    }
  }    

FUNCTION evaluate_objective_function
//This function defines objective function value to be minimized

//Calculate residual sums of squares
  //For mean length-at-age component
  rss_length=norm2(log(obs_length)-log(pred_length));

  //For asymptotic length deviations component  
  rss_Linf=norm2(log_Linf_devs);  

//Minimize the weighted negative log concentrated likelihood
  c_like=((n_length+n_Linf_devs)/2.0)*log(rss_length+lambda*rss_Linf);
 

REPORT_SECTION
//Output to "growth_time.rep" file
  report << "Annual asymptotic lengths (Linf), years 2-10" << endl;
  report << Linf << endl;
  report << "Growth coefficient (K)" << endl;
  report << K << endl;
  report << "Observed mean length-at-age matrix, years 1-10, ages 1-10" << endl;
  report << obs_length << endl;
  report << "Predicted mean length-at-age matrix, years 1-10, ages 1-10" << endl;
  report << pred_length << endl;
    
  
