//Simulate observed data for statistical catch-at-age analysis
//For ADMB short course 2, August 2007
//Created by B. Linton

DATA_SECTION
//Read in data from simulate.dat
  init_int fyear  //first year
  init_int lyear  //last year
  init_int fage  //first age
  init_int lage  //last age
  init_int nages_plus //number of ages to create plus group in first year
  init_number recruit_avg  //average recruitment
  init_number log_recruit_sd  //sd of log recruitment
  init_number M  //natural mortality
  init_vector effort(fyear,lyear)  //fishing effort 
  init_vector select(fage,lage)  //selectivity
  init_number q  //catchabitlity
  init_number log_q_sd  //sd of log catchability
  init_number log_total_catch_sd  //sd of log total catch
  init_number Nsample  //effective sample size for catch age composition
  init_int rnseed  //random number seed
  init_vector test(1,3)  //data entry test values

//Calculated quantities
  vector Nfyear(fage,nages_plus)  //recruitments for numbers at age in first year
  vector Nfyear_dev(fage,nages_plus)  //deviations in recruitments for numbers at age in first year
  vector Ffyear(fage,nages_plus)  //cumulative fishing mortality for numbers at age in first year
  vector Zfyear(fage,nages_plus)  //cumulative total mortality for numbers at age in first year
  vector qfyear_dev(fage,nages_plus-1)  //deviations in catchability in first year
  vector recruit_dev(fyear+1,lyear)  //deviations in recruitment
  matrix N(fyear,lyear,fage,lage)  //numbers at age
  matrix Z(fyear,lyear,fage,lage)  //total mortality
  matrix S(fyear,lyear,fage,lage)  //survival
  matrix F(fyear,lyear,fage,lage)  //fishing mortality
  vector q_dev(fyear,lyear)  //deviations in catchability
  matrix CAA(fyear,lyear,fage,lage)  //fishery catch at age
  vector total_catch(fyear,lyear)  //total annual catch
  matrix prop_CAA(fyear,lyear,fage,lage)  //proportion of catch at age
  vector total_catch_dev(fyear,lyear)  //deviations for measurement error in total catch
  matrix prop_CAA_dev(fyear,lyear,1,Nsample)  //deviations for multinomial error in proportion of catch at age
  vector total_catch_obs(fyear,lyear)  //observed total catch
  matrix CAA_obs(fyear,lyear,fage,lage)  //observed catch at age
  matrix prop_CAA_obs(fyear,lyear,fage,lage)  //observed proportion of catch at age
  number lambda  //ratio of relative variances of total catch to effort
  
//Declare looping variables
  int i  //loop over years
  int j  //loop over ages
  int k  //loop over replicates
  int l  //loop over levels of log total catch sd
  
//Check that data reads in correctly
  //!!cout << test << endl;
  //!!exit(5);

PARAMETER_SECTION
  //Need one active parameter for model to run
  init_number dummy
  //Need objective function value for model to compile
  objective_function_value fake

PRELIMINARY_CALCS_SECTION
//Following functions generate catch.dat
  get_random_numbers();
  get_mortality();
  get_numbers();
  get_catch();
  get_datafile();

//Exit before ADMB tries to estimate dummy parameter
  exit(0);

PROCEDURE_SECTION
//Define functions used in preliminary calcs section

FUNCTION get_random_numbers
//This function generates random numbers for simulated data
  random_number_generator rng(rnseed);  //initialize random number generator

//Fill vectors with standard normal random numbers
  Nfyear_dev.fill_randn(rng);  //deviations in recruitments for numbers at age in first year
  qfyear_dev.fill_randn(rng);  //deviations in catchability in first year 
  recruit_dev.fill_randn(rng);  //deviations in recruitment 
  q_dev.fill_randn(rng);  //deviations in catchability  

//Fill matrix with uniform random numbers
  prop_CAA_dev.fill_randu(rng);  //proportions of catch-at-age

FUNCTION get_mortality
//This function calculate mortality rates for simulated data
//Set value for total mortality for numbers at age in first year
  Zfyear=0.0;
  //Double loop calculates total mortlaity for numbers at age in first year
  for (j=fage+1;j<=nages_plus;j++)
  {
    if (j<=lage)
    {
      //Cumulative fishing mortality for each cohort in first year
      Ffyear(j)=select(j-1)*q*exp(qfyear_dev(j-1)*log_q_sd)*effort(1);
      //Cumulative total mortality for each cohort in first year
      Zfyear(j)=Zfyear(j-1)+Ffyear(j)+M;
    }
    if (j>=lage+1)  
    {
      //Cumulative fishing mortality for ages in plus group
      Ffyear(j)=select(lage)*q*exp(qfyear_dev(j-1)*log_q_sd)*effort(1);
      //Cumulative total mortality for ages in plus group
      Zfyear(j)=Zfyear(j-1)+Ffyear(j)+M;
    }
  }    

  //Loop calculates fishing mortality
  for (i=fyear;i<=lyear;i++)
  {
    F(i)=select*q*effort(i)*exp(q_dev(i)*log_q_sd);  
  }

  //Calculate total mortality
  Z=F+M;
  //Calculate survival
  S=exp(-1.0*Z);

FUNCTION get_numbers
//This function calculates numbers of fish at age
//Calculate number of recruits for each cohort in first year
  Nfyear=recruit_avg*exp(Nfyear_dev*log_recruit_sd);

//Loop calculates numbers of fish at age in first year
  for (j=fage;j<=nages_plus;j++)
  {
    if (j<=lage)
    {
      N(1,j)=Nfyear(j)*exp(-1.0*Zfyear(j));  //numbers at age in first year
    }
    if (j>=lage+1)
    {
      N(1,lage)+=Nfyear(j)*exp(-1.0*Zfyear(j));  //numbers at age for plus group
    }
  }

//Double loop fills in numbers at age in subsequent years
  for (i=fyear;i<=lyear-1;i++)
  {
    //Calculate number of recruits
    N(i+1,1)=recruit_avg*exp(recruit_dev(i+1)*log_recruit_sd); 
    for (j=fage;j<=lage-2;j++)
    {
      N(i+1,j+1)=N(i,j)*S(i,j);  //fill in rest of numbers at age
    }
    //Accumulate numbers at age in plus group
    N(i+1,lage)=N(i,lage)*S(i,lage)+N(i,lage-1)*S(i,lage-1);
  }

FUNCTION get_catch
//This function calculates catch-at-age and proportion of catch-at-age
//Calculate catch-at-age
  CAA=elem_prod(elem_div(F,Z),elem_prod(1.0-S,N));  //Baranov's catch equation

//Loop calculates total catch and proportion of catch-at-age
  for (i=fyear;i<=lyear;i++)
  {
    total_catch(i)=sum(CAA(i));  //total annual catch
    prop_CAA(i)=CAA(i)/total_catch(i);  //proportion of catch-at-age

    //Add multinomial error to get observed proportion of catch-at-age
    CAA_obs(i)=multinomial(Nsample, prop_CAA_dev(i), prop_CAA(i), lage);  //see multinomial function defined below
    prop_CAA_obs(i)=CAA_obs(i)/sum(CAA_obs(i));  //observed proportion of catch-at-age
  }

  //Calcualte observed total annual catch with lognormal error
  total_catch_obs=elem_prod(total_catch,exp(total_catch_dev*log_total_catch_sd));  

FUNCTION get_datafile
//This function creates catch.dat
//Calculate ratio of relative variances
  lambda=square(log_total_catch_sd/log_q_sd);

//Read out observed data to new dat file
  ofstream data("catch.dat");  //"data" command reads out to catch.dat
  data << "#Simulated observed data for catch.tpl" << endl;
  data << "#First and last year" << endl;
  data << " " << fyear << " " << lyear << endl;
  data << "#First and last age" << endl;
  data << " " << fage << " " << lage << endl;
  data << "#Natural mortality" << endl;
  data << " " << M << endl;
  data << "#Observed total annual fishery carch, years are row, ages columns" << endl;
  data << total_catch_obs << endl;
  data << "#Observed proportion of catch-at-age, years are row, ages columns" << endl;
  data << prop_CAA_obs << endl;
  data << "#Number of fish sampled each year for age composition" << endl;
  data << " " << Nsample << endl;
  data << "#Observed fishery effort (millions of ft of net)" << endl;
  data << " " << effort << endl;
  data << "#Ratio of relative variances of total catch to catchability deviations" << endl;
  data << " " << lambda << endl;
  data << "#Average recruitment starting value" << endl;
  data << " " << log(recruit_avg) << endl;
  data << "#Catchability starting value" << endl;
  data << " " << log(q) << endl;
  data << "#SD of log total catch starting value" << endl;
  data << " " << log(log_total_catch_sd) << endl;
  data << "#Data entry test values" << endl;
  data << test << endl; 

//Read out true data to new dat file
  ofstream real("true.dat");  //"data" command reads out to true.dat
  real << "#Simulated true data for catch.tpl" << endl;
  real << "Population abundance (numbers of fish), years " << fyear << "-" << lyear << ", ages " << fage << "-" << lage << endl;
  real << N << endl;
  real << "Total mortality, years " << fyear << "-" << lyear << ", ages " << fage << "-" << lage << endl;
  real << Z << endl;

FUNCTION dvector multinomial(long N, dvector rand_uniform, dvector p, int dim_p)
  //Function created by M. Wilberg to generate multinomial observed catch-at-age
  if (0.999999>sum(p)>1.000001) 
  {
    cout << "vector of proportions does not sum to one for multinomial random sample" << endl; 
    exit(1);
  } 
  //define local variables to use in this function
  dvector sample(1,dim_p);  //create vector for sample from multinomial
  double p1;
  double p2;
  int i;
  int j;
  sample.initialize();
  rand_uniform=sort(rand_uniform);  //sort random numbers
  p1=0;
  p2=p(1);
  j=1;
  for (i=1;i<=dim_p;i++)
  {
    sample(i)=0;
    //assign random uniform numbers to proper bins for multinomial
    while (j <= N && rand_uniform(j) > p1 && rand_uniform(j) <= p2)
    {
      sample(i)++;
      j++;
    }
    if (i < dim_p)
    {
      p1=p2;
      p2+=p(i+1);
    }
  }
  return(sample);


  

  
